/***************************************************************************
 *   Copyright (C) 2004 by KSmoothDock team                                *
 *   dangvd@yahoo.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <kdialog.h>
#include <klocale.h>

#include <qvariant.h>
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <dcopclient.h>
#include <kapplication.h>

#include "confkickerdialog.h"

/*
 *  Constructs a ConfKickerDialog as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
ConfKickerDialog::ConfKickerDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "ConfKickerDialog" );

    textLabel1 = new QLabel( this, "textLabel1" );
    textLabel1->setGeometry( QRect( 30, 10, 500, 40 ) );

    textLabel2 = new QLabel( this, "textLabel2" );
    textLabel2->setGeometry( QRect( 30, 50, 460, 60 ) );

    buttonGroup2 = new QButtonGroup( this, "buttonGroup2" );
    buttonGroup2->setGeometry( QRect( 20, 120, 470, 90 ) );
    buttonGroup2->setFrameShadow( QButtonGroup::Plain );
    buttonGroup2->setLineWidth( 0 );

    textLabel4 = new QLabel( buttonGroup2, "textLabel4" );
    textLabel4->setGeometry( QRect( 50, 30, 281, 61 ) );

    textLabel3 = new QLabel( buttonGroup2, "textLabel3" );
    textLabel3->setGeometry( QRect( 50, 0, 410, 40 ) );

    rbYes = new QRadioButton( buttonGroup2, "rbYes" );
    rbYes->setGeometry( QRect( 20, 10, 16, 16 ) );
    rbYes->setChecked( TRUE );

    rbNo = new QRadioButton( buttonGroup2, "rbNo" );
    rbNo->setGeometry( QRect( 20, 50, 20, 20 ) );

    pushButton3 = new QPushButton( this, "pushButton3" );
    pushButton3->setGeometry( QRect( 200, 210, 121, 41 ) );
    languageChange();
    resize( QSize(511, 275).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

    // signals and slots connections
    connect( pushButton3, SIGNAL( clicked() ), this, SLOT( pushButton3_clicked() ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
ConfKickerDialog::~ConfKickerDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void ConfKickerDialog::languageChange()
{
    setCaption( tr2i18n( "Attention!" ) );
    textLabel1->setText( tr2i18n( "<font color=\"#0000ff\">It seems you are running your kicker panel at the bottom of the screen.</font>" ) );
    textLabel2->setText( tr2i18n( "<font color=\"#0000ff\">We recommend that you keep it at other positions (preferably at the top) of the screen, while you are running KSmoothDock. Do you want KSmoothDock to move it to the top automatically for you?</font>" ) );
    buttonGroup2->setTitle( QString::null );
    textLabel4->setText( tr2i18n( "No, don't move the kicker panel." ) );
    textLabel3->setText( tr2i18n( "Yes, let KSmoothDock move your kicker panel to the top for you." ) );
    rbYes->setText( tr2i18n( "radioButton3" ) );
    rbNo->setText( tr2i18n( "radioButton4" ) );
    pushButton3->setText( tr2i18n( "&OK" ) );
    pushButton3->setAccel( QKeySequence( tr2i18n( "Alt+O" ) ) );
}

void ConfKickerDialog::pushButton3_clicked()
{
    if (rbYes->isChecked()) {
			DCOPClient *client = KApplication::dcopClient();
			if (!client->isAttached()) {
				client->attach();
				client->registerAs("KSmoothDock");
			}

			QByteArray data;
			QDataStream args(data, IO_WriteOnly);
			args << 2; // 2 means TOP OF THE SCREEN
			client->send("kicker", "Panel", "setPanelPosition(int)", data );	
			accept();
		} else if (rbNo->isChecked()) {
			reject();
		}
}

#include "confkickerdialog.moc"
