/***************************************************************************
                          item.cpp  -  description
                             -------------------
    begin                : Fri May 23 2003
    copyright            : (C) 2003 by KSmoothDock team
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//#include <iostream.h>

#include <kglobal.h>
#include <kicontheme.h>

#include <qfile.h>
#include <qimage.h>
#include <qtextstream.h>
  
#include "item.h"

Item::Item(QString iconName, QString cmd, QString name, int minSize, int maxSize) {
  iLoader = KGlobal::iconLoader();
  command = cmd;
  this->name = name;

  this->minSize = minSize;
  this->maxSize = maxSize;
  this->iconName = iconName;
	fHasIcon = true;

  QImage icon(iLoader->loadIcon(iconName, KIcon::NoGroup, 64).convertToImage());
  if (minSize <= maxSize) {
    int count = maxSize - minSize + 1;

    icons.setAutoDelete(TRUE);

    for (int i = 0; i < count; i++) {
      if (i % SIZE_INC == 0)
        icons.append(new QPixmap(icon.smoothScale(minSize + i, minSize + i)));
    }
  }

  id = 0;

//  cout << command.latin1() << endl;
//  cout << iconName.latin1() << endl;
//  cout << name.latin1() << endl;
}

Item::Item(QString fileName, int minSize, int maxSize) {
  QString line;
  QFile in(fileName);
  in.open(IO_ReadOnly);

  if (in.readLine(line, MAX_LEN) != -1) {
    if (line.contains("[Desktop Entry]", FALSE) > 0) {
      while (in.readLine(line, MAX_LEN) != -1) {
        if (line.startsWith("Exec=")) {
          command = line.mid(5).stripWhiteSpace();
        } else if (line.startsWith("Icon=")) {
          iconName = line.mid(5).stripWhiteSpace();
        } else if (line.startsWith("Name=")) {
          name = line.mid(5).stripWhiteSpace();
        }
      }
    }
  }

  in.close();

  iLoader = KGlobal::iconLoader();
  this->minSize = minSize;
  this->maxSize = maxSize;
	fHasIcon = true;

  QImage icon(iLoader->loadIcon(iconName, KIcon::NoGroup, 64).convertToImage());
  if (minSize <= maxSize) {
    int count = maxSize - minSize + 1;

    icons.setAutoDelete(TRUE);

    for (int i = 0; i < count; i++) {
      if (i % SIZE_INC == 0)
        icons.append(new QPixmap(icon.smoothScale(minSize + i, minSize + i)));
    }
  }

  id = 0;

//  cout << command.latin1() << endl;
//  cout << iconName.latin1() << endl;
//  cout << name.latin1() << endl;
}

/*
Item::Item(QString fileName) {
  QString line;
  QFile in(fileName);
  in.open(IO_ReadOnly);

  if (in.readLine(line, MAX_LEN) != -1) {
    if (line.contains("[Desktop Entry]", FALSE) > 0) {
      while (in.readLine(line, MAX_LEN) != -1) {
        if (line.startsWith("Exec=")) {
          command = line.mid(5).stripWhiteSpace();
        } else if (line.startsWith("Icon=")) {
          iconName = line.mid(5).stripWhiteSpace();
        } else if (line.startsWith("Name=")) {
          name = line.mid(5).stripWhiteSpace();
        }
      }
    }
  }

  in.close();
	id = 0;
}
*/

Item::Item(QPixmap iconBig, WId id, QString name, int minSize, int maxSize) {
  this->id = id;
  this->name = name;
  this->minSize = minSize;
  this->maxSize = maxSize;
  command = "";
	iconName = "";
	iLoader = KGlobal::iconLoader();
	fHasIcon = true;
	
	int orgW, orgH; // original width and height	
	orgW = iconBig.width();
	orgH = iconBig.height();

  QImage icon(iconBig.convertToImage());
  if (minSize <= maxSize) {
    int count = maxSize - minSize + 1;

    icons.setAutoDelete(TRUE);

    for (int i = 0; i < count; i++) {
      if (i % SIZE_INC == 0)
        icons.append(new QPixmap(icon.smoothScale(minSize + i, (minSize + i)*orgH/orgW))); // scale to fit width
    }
  }
}

Item::Item(QString cmd, QString name, int minSize, int maxSize) {
  iLoader = KGlobal::iconLoader();
  command = cmd;
  this->name = name;

  this->minSize = minSize;
  this->maxSize = maxSize;
  this->iconName = "";
	fHasIcon = false;
	id = 0;
}

Item::~Item() {
}

bool Item::hasIcon() {
	return fHasIcon;
}

QPixmap* Item::getIcon(int size) {
  if (fHasIcon && (size >= minSize) && (size <= maxSize)) {
    int d = size - minSize;
    return icons.at(d / SIZE_INC);
  }
  else
    return 0;
}

QString Item::getCommand() {
  return command;
}

QString Item::getName() {
  return name;
}

QString Item::getIconName() {
	return iconName;
}

WId Item::getId() {
  return id;
}

void Item::setIcon(QPixmap iconBig) {
	fHasIcon = true;
  icons.clear();
  QImage icon(iconBig.convertToImage());

  int count = maxSize - minSize + 1;
	
	int orgW, orgH; // original width and height	
	orgW = iconBig.width();
	orgH = iconBig.height();

  for (int i = 0; i < count; i++) {
    if (i % SIZE_INC == 0)
      icons.append(new QPixmap(icon.smoothScale(minSize + i, (minSize + i)*orgH/orgW)));
  }
}

void Item::setName(QString newName) {
  name = newName;
}

void Item::setCommand(QString cmd) {
	command = cmd;
}

void Item::setIconName(QString iName) {
	iconName = iName;
	QPixmap pix = iLoader->loadIcon(iconName, KIcon::NoGroup, 64);
	setIcon(pix);
}

void Item::saveToFile(QString fileName) {
  QFile out(fileName);
  if (out.open(IO_WriteOnly)) {
    QTextStream out_s(&out);
    out_s << "[Desktop Entry]" << endl;
    out_s << "Comment=" << endl;
    out_s << "Comment[en_GB]=" << endl;
    out_s << "Encoding=UTF-8" << endl;
    out_s << "Exec=" << command << endl;
    out_s << "GenericName=" << endl;
    out_s << "GenericName[en_GB]=" << endl;
    out_s << "Icon=" << iconName << endl;
    out_s << "MimeType=" << endl;
    out_s << "Name=" << name << endl;
    out_s << "Name[en_GB]=" << endl;
    out_s << "Path=" << endl;
    out_s << "ServiceTypes=" << endl;
    out_s << "SwallowExec=" << endl;
    out_s << "SwallowTitle=" << endl;
    out_s << "Terminal=false" << endl;
    out_s << "TerminalOptions=" << endl;
    out_s << "Type=Application" << endl;
    out_s << "X-KDE-SubstituteUID=false" << endl;
    out_s << "X-KDE-Username=" << endl;

    out.close();
  }
}
