/***************************************************************************
 *   Copyright (C) 2004 by KSmoothDock team                                *
 *   dangvd@yahoo.com                                                      *
 *   Based on xosd.cpp (C) 2004 by KoolDock team                           * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "tooltip.h"

#include <iostream.h>

#include <qpainter.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qstring.h>
#include <qfont.h>
#include <qfontmetrics.h>

#include <qwidget.h>
#include <klocale.h>
#include <netwm.h>
#include <netwm_def.h>
#include <kwin.h>

#include <kdebug.h>

ToolTip::ToolTip(QWidget *parent, const char *name) : QWidget(parent, name, WStyle_Customize |  WRepaintNoErase | WStyle_NoBorder | WDestructiveClose | WResizeNoErase | WMouseNoMask | WStyle_StaysOnTop | WX11BypassWM)
{
   info = new NETWinInfo( qt_xdisplay(), winId(), qt_xrootwin(), NET::WMState);
   info->setDesktop( NETWinInfo::OnAllDesktops );
   KWin::setState(winId(), NET::SkipTaskbar | NET::SkipPager | NET::KeepAbove);
}

ToolTip::~ToolTip()
{
}

void ToolTip::setColor(QColor color)
{
   fontColor = color;
}

void ToolTip::setBgColor(QColor color) {
	bgColor = color;
}

void ToolTip::setText(QString t)
{
   text = t;
	 doUpdateGeometry();
}

void ToolTip::doUpdateGeometry() {
   QFontMetrics *metrics = new QFontMetrics(font);
   w = metrics->width(text);
   w = w + 5; // 5 pixels more
   h = metrics->height();
	
   resize(w,h);	 
   update();
}

void ToolTip::setFontFace(QString fontFace)
{
   font.setFamily(fontFace);
}

void ToolTip::setItalic(bool val)
{
   font.setItalic(val);
}

void ToolTip::setBold(bool val)
{
   font.setBold(val);
}

void ToolTip::setSize(int size)
{
   font.setPointSize(size);
   fontSize = size;
}

void ToolTip::paintEvent(QPaintEvent *)
{
	QPixmap pm(size());
	QBitmap bm(size());
	
	// draw the mask bitmap
	
	QPainter p0;
	p0.begin(&bm);
	
	p0.fillRect(rect(), Qt::color0);
	
	p0.setPen( Qt::color1 );
	p0.setFont(font);
	
	for (int i = -2; i <= 2; i++)
	for (int j = -2; j <= 2; j++) {
		p0.drawText(1 + i, fontSize + j, text);
	}   

	p0.drawText(1, fontSize, text);

	p0.end();
	
	// draw the buffer pixmap
	
	QPainter p;
	p.begin(&pm);   
	
	p.setPen( bgColor );
	p.setFont(font);
	
	for (int i = -2; i <= 2; i++)
	for (int j = -2; j <= 2; j++) {
		p.drawText(1 + i, fontSize + j, text);
	}   

	// now, draw normal text
	p.setPen( fontColor );
	p.setFont(font);
	p.drawText(1, fontSize, text);

	p.end();
	
	// update the widget
	
	bitBlt(this, 0, 0, &pm);
	info->setState(NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager, NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager);
	setMask(bm); // for transparency
}

#include "tooltip.moc"
